/*

Arquivo:

   ST.cpp

Descrio:

   Implementao das funes em ST.hpp.

*/

//
// Cabealhos
//
#include <stdio.h>
#include "ST.hpp"

//
// Tipos de Dados
//

struct STable::node {
  Item* item;
  node* left;
  node* right;
  
  node(Item* it, node* l = NULL, node* r = NULL) 
    : item(it), left(l), right(r) { }
  ~node() { delete item; }
};

//
// Funes Privadas
//

/*
 * insertR
 *   Adiciona um elemento na rvore.
 *
 * Argumentos:
 * [in]  root                 Raiz da rvore
 *       item                 Elemento
 *
 * Retornos:
 *   Ponteiro para o novo n criado.
 *
 * Excees:
 *   std:bad_alloc
 *
 */
STable::node* STable::insertR(node* root, Item* item)
{
  if(root == NULL)
    return new node(item, NULL, NULL);

  int ret = root->item->compare(*item);

  if(ret < 0) // root < item
    root->right = insertR(root->right, item);
  else
    root->left = insertR(root->left, item);

  return root;
}

/*
 * searchR
 *   Procura por um elemento na rvore e retorna um ponteiro para ele se o 
 * encontrar
 *
 * Argumentos:
 * [in]  root                 Raiz da rvore a ser varrida
 *       key                  Chave
 *
 * Retornos:
 *   Ponteiro para o elemento, se o encontrar, ou NULL caso contrrio.
 *
 */
Item* STable::searchR(node* root, const char* key) const
{
  if(root == NULL)
    return NULL;

  int ret = root->item->compare(key);

  if(ret < 0) // root < key
    return searchR(root->right, key);
  else if(ret > 0)
    return searchR(root->left, key);
  else
    return root->item;
}

/*
 * deleteR
 *   Apaga todos os elementos de uma rvore de raiz root.
 *
 * Argumentos:
 * [in]  root                 rvore da raiz a ser apagada.
 *
 */
void STable::deleteR(node* root)
{
  if(root == NULL)
    return;

  /* Apaga as subrvores esquerda e direita recursivamente */
  deleteR(root->left);
  deleteR(root->right);
  
  /* Apaga a raiz */
  delete root;
}

/*
 * sortR
 *   Percorre a rvore in-ordem.
 *
 * Argumentos:
 * [in]  root                 Raiz da rvore
 *       visit                Funo que  chamada quando um elemento  visitado
 *
 */
void STable::sortR(node* root, void (*visit)(Item&)) const
{
  if(root == NULL)
    return;

  sortR(root->left, visit);
  (*visit)(*root->item);
  sortR(root->right, visit);
}
