/*

Arquivo:

   ST.cpp

Descrio:

   Implementao das funes em ST.hpp.

*/

//
// Cabealhos
//
#include <stdio.h>
#include "ST.hpp"

//
// Tipos de Dados
//

struct STable::node {
  Item* item;
  node* left;
  node* right;
  int rank;
  
  node(Item* it, node* l = NULL, node* r = NULL, int rk = 0) 
    : item(it), left(l), right(r), rank(rk) { }
  ~node() { delete item; }
};

//
// Funes Privadas
//

/* Rotaes */
STable::node* STable::rotR(node* h)
{ 
  node* x = h->left; 
  h->left = x->right;
  x->right = h; 
  return x; 
}

STable::node* STable::rotL(node* h)
{ 
  node* x = h->right; 
  h->right = x->left; 
  x->left = h; 
  return x;
}

/*
 * insert_treapR
 *   Insero de um elemento no treap.
 *
 */
STable::node* STable::insert_treapR(node* root, Item* item, int rank)
{
  if(root == NULL)
    return new node(item, NULL, NULL, rank);

  int ret = root->item->compare(*item);

  if(ret < 0) { // root < item
    root->right = insert_treapR(root->right, item, rank);

    /* Conserta possvel problema com o rank */
    if(root->right->rank < root->rank)
      root = rotL(root);
  }
  else {
    root->left = insert_treapR(root->left, item, rank);
    
    /* Conserta... */
    if(root->left->rank < root->rank)
      root = rotR(root);
  }

  return root;
}

/*
 * searchR
 *   Procura por um elemento na rvore e retorna um ponteiro para ele se o 
 * encontrar
 *
 * Argumentos:
 * [in]  root                 Raiz da rvore a ser varrida
 *       key                  Chave
 *
 * Retornos:
 *   Ponteiro para o elemento, se o encontrar, ou NULL caso contrrio.
 *
 */
Item* STable::searchR(node* root, const char* key) const
{
  if(root == NULL)
    return NULL;

  int ret = root->item->compare(key);

  if(ret < 0) // root < key
    return searchR(root->right, key);
  else if(ret > 0)
    return searchR(root->left, key);
  else
    return root->item;
}

/*
 * deleteR
 *   Apaga todos os elementos de uma rvore de raiz root.
 *
 * Argumentos:
 * [in]  root                 rvore da raiz a ser apagada.
 *
 */
void STable::deleteR(node* root)
{
  if(root == NULL)
    return;

  /* Apaga as subrvores esquerda e direita recursivamente */
  deleteR(root->left);
  deleteR(root->right);
  
  /* Apaga a raiz */
  delete root;
}

/*
 * sortR
 *   Percorre a rvore in-ordem.
 *
 * Argumentos:
 * [in]  root                 Raiz da rvore
 *       visit                Funo que  chamada quando um elemento  visitado
 *
 */
void STable::sortR(node* root, void (*visit)(Item&)) const
{
  if(root == NULL)
    return;

  sortR(root->left, visit);
  (*visit)(*root->item);
  sortR(root->right, visit);
}
