/*

Arquivo:

   main.cpp

Descrio:

   Mdulo principal.

*/

//
// Incluso de Cabealhos
//
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include "input.hpp"
#include "mem.hpp"
#include "Item.hpp"
#include "ST.hpp"

//
// Defines
//
#define SUCCESS               0
#define ERROR_CMD_LINE        1
#define ERROR_OUT_OF_MEMORY   2
#define ERROR_OPEN_DICS       3

//
// Variveis Globais
//
static int num_dics = 0;
static mem_array<dic> dic_files = NULL;

//
// Prottipos
//
int scan_command_line(int, char* []);
void visit(Item&);

//
// Funo Main
//
int main(int argc, char* argv[])
{
  input_obj in_obj;

  /* Procura pelos argumentos de linha de comando */
  int ret = scan_command_line(argc, argv);

  if(ret == -1)
    return ERROR_CMD_LINE;

  /* Abre os dicionrios */
  if(argc - ret > 0) {
    num_dics = argc - ret;

    try {
      dic_files = new dic[num_dics];
    }
    catch(std::bad_alloc& m) {
      cerr << "Erro de alocao de memria" << endl;
      return ERROR_OUT_OF_MEMORY;
    }

    for(int i = 0, j = ret; j < argc; i++, j++) {
      try {
	dic_files[i].open_file(argv[j]);
      }
      catch(dic::file_not_found) {
	cerr << "Arquivo " << argv[j] << " no encontrado" << endl;
	return ERROR_OPEN_DICS;
      }
    }
  }

  double start = clock();
  /* L a entrada padro e insere as palavras numa ABB */
  STable symbol_table;
  const char* word;
  while((word = input_obj::get_word(stdin)) != NULL) {
    if(symbol_table.search(word) == NULL) {
      try {
	mem_ptr<Item> new_item = new Item(word);
	symbol_table.insert(new_item);
	new_item.release();
      }
      catch(std::bad_alloc& m) {
	cerr << "Erro de alocao de memria" << endl;
	return ERROR_OUT_OF_MEMORY;
      }
    }
  }

  /* Faz a verificao ortogrfica */
  symbol_table.sort(&visit);

  double elapsed = (clock() - start) / CLOCKS_PER_SEC;

  cerr << endl << "Tempo de processamento: " << elapsed << endl;

  return 0;
}

//
// Funes
//

/*
 * scan_command_line
 *   Verifica os argumentos da linha de comando e aplica seus efeitos.
 *
 * Argumentos:
 * [in]  argc, argv           Parmetros passados a funo main
 *
 * Retornos:
 *   Retorna o ndice do primeiro dicionrio encontrado ou -1 se ocorrer
 * um erro.
 *
 */
int scan_command_line(int argc, char* argv[])
{
  int delta = 0;
  int maxl = 50;
  int temp;

  /* Inicializao */
  for(int i = 0; i < 256; i++) input_obj::set_ord(i, i);

  /* Verificao */
  int i;
  for(i = 1; i < argc; i++) {
    if(argv[i][0] != '-')
      break;

    switch(argv[i][1]) {
    case 'a':
      for(int j = 1; argv[i][j+1] != '\0'; j++)
	input_obj::set_ord(argv[i][j+1], j+delta);
      break;
    case 'b':
      if(argv[i][2] == '\0')
	for(int j = 0; j < 256; input_obj::set_ord(j++, input_obj::ord_break));
      else
	for(int j = 2; argv[i][j] != '\0'; 
	    input_obj::set_ord(argv[i][j++], input_obj::ord_break));
      break;
    case 'n':
      if(argv[i][2] == '\0')
	for(int j = 0; j < 256; input_obj::set_ord(j++, input_obj::ord_null));
      else
	for(int j = 2; argv[i][j] != '\0'; 
	    input_obj::set_ord(argv[i][j++], input_obj::ord_null));
      break;
    case 'd':
      temp = atoi(argv[i] + 2);
      if(temp < 0 || temp > 255) {
	cerr << "Valor invlido seguindo opo -d" << endl;
	return -1;
      }
      delta = temp;
      break;
    case 'm':
      temp = atoi(argv[i] + 2);
      if(temp < 50 || temp > 1000) {
	cerr << "Valor invlido seguindo opo -m" << endl;
	return -1;
      }
      maxl = temp;
      break;
    case 'T':
      for(int j = 1; j < 256; j++) {
	if(isspace(j) || ispunct(j))
	  input_obj::set_ord(j, input_obj::ord_break);
      }
      break;
    case 't':
      for(int j = 1; j < 256; j++) {
	if(isspace(j))
	  input_obj::set_ord(j, input_obj::ord_break);
      }
      break;
    default:
      cerr << "Argumento invlido: " << argv[i] << endl;
      return -1;
    }
  }

  try {
    input_obj::alloc_buffer(maxl);
  }
  catch(std::bad_alloc& m) {
    cerr << "Erro de alocao de memria" << endl;
    return -1;
  }

  if(input_obj::get_ord('\n') == input_obj::ord_break)
    breakchar = '\n';
  else {
    breakchar = '\0';
    for(int i = 255; i > 0; i--)
      if(input_obj::get_ord(i) == input_obj::ord_break)
	breakchar = i;
  }

  if(breakchar == '\0')
    input_obj::set_default();

  return i;
}

/*
 * visit
 *   Callback para o mtodo sort de STable.
 *
 */
void visit(Item& item)
{
  bool found = false;
  
  /* Verifica os dicionrios */
  for(int i = 0; i < num_dics; i++) {
    /* Enquanto a palavra lida for maior que a atual do dicionrio */
    int ret = 0;
    while(!dic_files[i].eof() && 
	  (ret = input_obj::wordcmp(item.get_word(), 
				    dic_files[i].curr_word())) > 0)
      dic_files[i].next();

    /* Encontrou a palavra? */
    if(!dic_files[i].eof() && ret == 0)
      found = true;
  }

  /* Imprime a palavra se foi encontrada */
  if(!found)
    input_obj::word_print(item.get_word());
}
