/*

Arquivo:

   input.cpp

Descrio:

   Implementao das funes e classes declaradas em input.hpp.

*/

//
// Incluso de cabealhos
//
#include <iostream.h>
#include <stdio.h>
#include <ctype.h>
#include "mem.hpp"
#include "input.hpp"

//
// Variveis Globais
//
unsigned char breakchar = '\n';

//
// Classes
//

/* Gerencimento de Entrada e Sada - Variveis Estticas */
int input_obj::ord[256];
char* input_obj::buffer = NULL;
int input_obj::buffer_size = 51;
int input_obj::max_word_length = 50;

/* Mtodos */

/*
 * Lana exceo std::bad_alloc
 *
 */
void input_obj::alloc_buffer(int maxl)
{
  /* Tamanho mximo da palavra e do buffer */
  buffer_size = maxl+1;
  max_word_length = maxl;

  /* Aloca espao para o buffer */
  buffer = new char[buffer_size];
}

/*
 * get_word
 *   L uma palavra de um arquivo apontado por src_file seguindo as convenes
 * determinadas para breaks, nulls, etc...
 *
 * Argumentos:
 * [in]  src_file             Arquivo de entrada, do qual ser lida a palavra
 *
 * Retornos:
 *   Ponteiro para a palavra lida, ou NULL se ocorrer algum erro ou fim de
 * arquivo
 *
 */
const char* input_obj::get_word(FILE* src_file)
{
  char c;

  /* L do arquivo at que um caractere vlido seja atingido */
  for(c = fgetc(src_file); !feof(src_file) && !is_valid(c);
      c = fgetc(src_file));

  if(feof(src_file))
    return NULL;

  /* Inicia a leitura do arquivo - o primeiro caractere j foi lido */
  int i = 0;
  while(!feof(src_file) && (is_valid(c) || is_null(c))) {
    /* Se o caractere lido foi nulo, ele  ignorado */
    if(is_null(c)) {
      c = fgetc(src_file);
      continue;
    }

    /* Caso contrrio, insere o caractere */
    buffer[i] = c;

    /* L o prximo caractere, se o tamanho da palavra ainda no estourou */
    i++;
    if(i == max_word_length)
      break;
    c = fgetc(src_file);
  }

  /* Se saimos por eof, no retornamos a palavra */
  if(feof(src_file))
    return NULL;

  /* Insere final nulo */
  buffer[i] = breakchar;

  /* Retorna */
  return buffer;
}

/*
 * set_default
 *   Utiliza ords default
 *
 */
void input_obj::set_default()
{
  /* Inicializa tabela de ords */
  for(int i = 'a'; i <= 'z'; ord[i++] = i - 'a' + 'A');
  ord['\n'] = ord_break;
  breakchar = '\n';
}

/*
 * wordcmp
 *   Compara duas palavras segundo a ordem dada pela tabela de ords.
 *
 * Argumentos:
 * [in]  word1, word2         Palavras a serem comparadas
 *
 * Retornos:
 *   Retorna um valor negativo se word1 < word2, nulo se word1 = word2 e positivo
 * se word1 > word2.
 *
 */
int input_obj::wordcmp(const char* word1, const char* word2)
{
  const byte *u, *v;
  for(u = (byte*) word1, v = (byte*) word2; 
      (*u != breakchar && *v != breakchar) && (ord[*u] == ord[*v]); u++, v++);

  if(*u == breakchar && *v == breakchar)
    return 0;
  else if(ord[*u] < ord[*v])
    return -1;
  else
    return 1;
}

/*
 * wordcpy
 *   Copia uma palavra para outra.
 *
 * Argumentos:
 * [in]  src                  Palavra de origem
 * [out] dest                 Buffer de destino
 *
 */
void input_obj::wordcpy(char* dest, const char* src)
{
  int i;
  for(i = 0; src[i] != breakchar; dest[i++] = src[i]);
  dest[i] = breakchar;
}

/*
 * worlen
 *   Retorna o tamanho de uma palavra
 *
 */
int input_obj::wordlen(const char* word)
{
  int i;
  for(i = 0; word[i] != breakchar; i++);
  return i;
}

/*
 * word_print
 *   Imprime uma sequncia de caracteres.
 *
 * Argumentos:
 * [in]  str                  Sequncia a ser impressa
 *
 */
void input_obj::word_print(const char* str)
{
  for(const char* c = str; *c != breakchar; c++)
      cout << *c;

  cout << breakchar;
}

/* Dicionrio - Construtores e Destrutores */

/* lana std::bad_alloc */
dic::dic()
{
  dic_file = NULL;
  word_size = input_obj::get_buffer_size();
  word = new char[word_size];
}

dic::~dic()
{
  if(dic_file != NULL)
    fclose(dic_file);
  delete[] word;
}

/* Mtodos */

/*
 * open_file
 *   Abre um arquivo de dicionrio e carrega sua primeira palavra.
 *
 * Argumentos:
 * [in]  file_name            Nome do arquivo.
 *
 * Excees:
 *   Se o arquivo no for encontrado, lana a exceo file_not_found.
 */
void dic::open_file(const char* file_name)
{
  /* Abre o arquivo e verifica erros */
  dic_file = fopen(file_name, "r");

  if(dic_file == NULL)
    throw file_not_found();

  /* Carrega a primeira palavra do arquivo - utiliza mtodo next */
  next();
}
    
/*
 * next
 *   L a prxima palavra do arquivo.
 *
 * Excees:
 *   std::bad_alloc.
 */
void dic::next()
{
  /* Faz a leitura da palavra */
  const char* ret = input_obj::get_word(dic_file);

  if(ret == NULL) return;

  /* Copia a palavra */
  input_obj::wordcpy(word, ret);
}
