/*

Arquivo:

   input.hpp

Descrio:

   Classe e funes para tratamento de entrada e outros.

*/

#ifndef _INPUT_HEADER_
#define _INPUT_HEADER_

//
// Incluso de Cabealhos
//
#include <stdio.h>

//
// Typedefs
//
typedef unsigned char byte;

//
// Variveis Globais
//
extern unsigned char breakchar;

//
// Classes
//

/* Gerenciamento de entrada / sada */
class input_obj {
  /* Vetor de ords */
  static int ord[256];

  /* Buffer de Leitura */
  static char* buffer;
  static int buffer_size;
  static int max_word_length;

public:
  /* Constantes */
  enum { ord_break = -1, ord_null = 0 };

  /* Construtores e Destrutores */
  input_obj() { }
  ~input_obj() { delete[] buffer; }

  /* Buffer */
  static int get_buffer_size() { return buffer_size; }

  /* Inicializao */
  static void alloc_buffer(int);

  /* Tabela de ords */
  static void set_default();
  static void set_ord(byte b, int o) { ord[b] = o; }
  static int get_ord(byte b) { return ord[b]; }
  static bool is_null(byte b) { return ord[b] == ord_null; }
  static bool is_break(byte b) { return ord[b] == ord_break; }
  static bool is_valid(byte b) { return ord[b] > 0; }

  /* Leitura e manipulao de texto */
  static const char* get_word(FILE*);
  static int wordcmp(const char*, const char*);
  static void wordcpy(char*, const char*);
  static int wordlen(const char*);
  static void word_print(const char*);
};

/* Dicionrio */
class dic {
  /* Arquivo do dicionrio */
  FILE* dic_file;

  /* Buffer para a palavra atual */
  char* word;
  int word_size;

public:
  /* Construtores e Destrutores */
  dic();
  ~dic();

  /* Mtodos */
  void open_file(const char*);
  bool eof() const { return feof(dic_file); }
  const char* curr_word() const { return word; }
  void next();

  /* Excees */
  class file_not_found { };
};

#endif
